'use strict';

var pg = require('pg');
var assert = require('assert');

function buildQuery(triggers) {
  assert.equal(typeof triggers, 'string');

  return '\n    CREATE OR REPLACE FUNCTION table_update_notify() RETURNS trigger AS $$\n    DECLARE\n      id BIGINT;\n      row RECORD;\n    BEGIN\n      IF TG_OP = \'INSERT\' OR TG_OP = \'UPDATE\' THEN\n        id = NEW.id;\n        row = NEW;\n      ELSE\n        id = OLD.id;\n        row = OLD;\n      END IF;\n      PERFORM pg_notify(\'table_update\', json_build_object(\'table\', TG_TABLE_NAME, \'id\', id, \'type\', TG_OP, \'row\', row_to_json(row))::text);\n      RETURN NEW;\n    END;\n    $$ LANGUAGE plpgsql;\n\n    ' + triggers + '\n  ';
}

function buildTriggers(tables) {
  return tables.map(function (tableName) {
    return '\n      DROP TRIGGER IF EXISTS ' + tableName + '_notify_update ON ' + tableName + ';\n      CREATE TRIGGER ' + tableName + '_notify_update AFTER UPDATE ON ' + tableName + ' FOR EACH ROW EXECUTE PROCEDURE table_update_notify();\n\n      DROP TRIGGER IF EXISTS ' + tableName + '_notify_insert ON ' + tableName + ';\n      CREATE TRIGGER ' + tableName + '_notify_insert AFTER INSERT ON ' + tableName + ' FOR EACH ROW EXECUTE PROCEDURE table_update_notify();\n\n      DROP TRIGGER IF EXISTS ' + tableName + '_notify_delete ON ' + tableName + ';\n      CREATE TRIGGER ' + tableName + '_notify_delete AFTER DELETE ON ' + tableName + ' FOR EACH ROW EXECUTE PROCEDURE table_update_notify();\n    ';
  }).join('');
}

module.exports = function (opts, cb) {
  assert(Array.isArray(opts.tables), 'opts.tables should be an array.');
  assert.ok(opts.db, 'need db connection string');

  // nothing to do
  if (!opts.tables.length) return cb(null, { message: 'nothing to do' });

  pg.connect(opts.db, function (err, client, done) {
    if (err) return cb(err);

    var triggers = buildTriggers(opts.tables);
    var query = client.query(buildQuery(triggers));

    // query.on('row', function(row) {
    //   console.log(row)
    // })

    query.on('error', function (queryErr) {
      done(queryErr);
      cb(queryErr);
    });

    query.on('end', function () {
      client.end();
      cb(null, { message: 'done' });
    });
  });
};

module.exports.buildQuery = buildQuery;
module.exports.buildTriggers = buildTriggers;